import { Link, Route, Routes, useParams } from "react-router-dom";
import useSWR from "swr";
import { fetchJson } from "../../../util/fetchJson.ts";
import { useAtom } from "jotai";
import { namespaceAtom, useNamespace } from "../../../atoms/namespace.ts";
import { Loader } from "../../../components/loader.tsx";
import { useEffect, useState } from "preact/hooks";
import { NewTagModal } from "./newTagModal.tsx";
import { ResourceList } from "../../resourceList.tsx";

export const TagRouter = () => {
  return (
    <Routes>
      <Route index Component={TagList} />
      <Route path=":typeTag" Component={TagEditor} />
    </Routes>
  );
};

function TagList() {
  const [namespace, _setNamespace] = useAtom(namespaceAtom);
  const { data, isLoading } = useSWR<string[]>(
    `/api/pack/${namespace}/tags`,
    fetchJson,
  );

  const [showNewTagModal, setShowNewTagModal] = useState(false);

  if (isLoading) {
    return <Loader msg="Geez, when was the last time you swept?" />;
  }

  return (
    <div>
      <ul>
      </ul>
      <ResourceList />
      {false && (
        <ul class="flex flex-col gap-2">
          <li>
            <button onClick={() => setShowNewTagModal(true)}>New Tag</button>
            {showNewTagModal && (
              <NewTagModal
                close={() => {
                  setShowNewTagModal(false);
                }}
              />
            )}
          </li>
          {data?.map((tag) => (
            <li class="flex gap-2">
              <Link to={`/editor/tags/${tag}`}>{tag}</Link>
              <button>Delete</button>
            </li>
          ))}
        </ul>
      )}
    </div>
  );
}

interface Tag {
  replace: boolean;
  values: (string | { id: string; required: boolean })[];
}

function TagEditor() {
  const [namespace, _setNamespace] = useNamespace();
  const { typeTag } = useParams();
  const { data, isLoading } = useSWR<Tag>(
    `/api/pack/${namespace}/tags/${typeTag}`,
    fetchJson,
  );
  const [tag, setTag] = useState<Tag>({
    replace: false,
    values: [],
  });

  useEffect(() => {
    if (isLoading) {
      return;
    }
    setTag(data || { replace: false, values: [] });
  }, [data, isLoading]);

  if (isLoading) {
    return <Loader msg="Your hard drive is full of... interesting things." />;
  }

  return (
    <div>
      <h3 class="text-lg font-bold">
        {typeTag}:
        <input
          type="checkbox"
          name="replace"
          checked={tag.replace}
          onChange={(e) =>
            setTag({ ...tag, replace: (e.target as any).checked })}
        />
        {
          /* {tag.values.map((value, i) => (
          <div class="flex gap-2">
            <input
              type="text"
              value={value}
              onInput={(e) =>
                setTag({
                  ...tag,
                  values: tag.values.map((v, i) =>
                    i === i ? e.target.value : v
                  ),
                })}
            />
            <button
              onClick={() =>
                setTag({
                  ...tag,
                  values: tag.values.filter((_, i) => i !== i),
                })}
            >
              Delete
            </button>
          </div>
        ))} */
        }
        {tag.values.length === 0 && (
          <button onClick={() => setTag({ ...tag, values: [""] })}>
            Add Value
          </button>
        )}
      </h3>
    </div>
  );
}

