import { useState } from "preact/hooks";
import { Modal } from "../../modal.tsx";
import { useNavigate } from "react-router-dom";
import { namespaceAtom } from "../../../atoms/namespace.ts";
import { useAtom } from "jotai";

export const NewTagModal = ({ close }: { close: () => void }) => {
  const [tagName, setTagName] = useState("");
  const [tagType, setTagType] = useState("block");
  const nav = useNavigate();
  const [namespace, _setNamespace] = useAtom(namespaceAtom);

  const createTag = async () => {
    const res = await fetch(
      `/api/pack/${namespace}/tags`,
      {
        method: "POST",
        body: JSON.stringify({
          tag: tagName,
          type: tagType,
        }),
      },
    );

    if (res.status === 200) {
      close();
      nav(`/editor/tags/${tagType}-${tagName}`);
    }
  };
  return (
    <Modal>
      <p class="mb-2">Create a new tag</p>
      <form
        class="flex flex-col gap-2"
        onSubmit={(e) => {
          e.preventDefault();
          createTag();
        }}
      >
        <label class="flex gap-2 items-center">
          Tag Type{" "}
          <select
            class="flex-1"
            name="tag-type"
            value={tagType}
            onChange={(e) => setTagType((e.target as any).value)}
          >
            <option value="block">Block</option>
            <option value="entity">Entity</option>
            <option value="item">Item</option>
            <option value="function">Function</option>
          </select>
        </label>
        <div class="flex gap-2">
          <input
            type="text"
            value={tagName}
            onInput={(e) => setTagName((e.target as any).value)}
            placeholder="Tag name"
          />
          <button type="submit">Create</button>
        </div>
      </form>
    </Modal>
  );
};
